using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Microsoft.Win32;

namespace WMV_PowerToy
{

    public enum vc1profile { AP, MP, SP };
        
    public partial class MainWindow : Form
    {

        Dictionary<string, WMVSetting> EncoderSettings = new Dictionary<string, WMVSetting>();
        Dictionary<string, WMVSetting> DecoderSettings = new Dictionary<string, WMVSetting>();

        string[] encoders = new string[5];
        string[] decoders = new string[2];
        Int32[] MapPercOptToADZ = new Int32[24] {0,0,1,1,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
        Int32[] MapADZToPercOpt = new Int32[21] {0,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};

        public MainWindow()
        {

            encoders[0] = "{96b57cdd-8966-410c-bb1f-c97eea765c04}"; // wmvdmoe2.dll - Simple and Main Profile in WMF9.5
            encoders[1] = "{44653d0d-8cca-41e7-baca-884337b747ac}"; // wvc1dmoe.dll - Advanced Profile beta
            encoders[2] = "{d23b90d0-144f-46bd-841d-59e4eb19dc59}"; // wmvencod.dll - SP, MP and AP in WMF11
            encoders[3] = "{8a9c3e48-5e81-4bc6-87eb-08bf2495acba}"; // tararidmo.dll, wmvaccel.dll - Released to ISVs, with support for HW acceleration
            encoders[4] = "{fd2a70ea-5c3f-4868-bf88-577e2821f8f8}"; // wmvebox.dll, vc1esdkdmo.dll - Released to ISVs, with extended APIs

            decoders[0] = "{82d353df-90bd-4382-8bc2-3f6192b76e34}"; // wmvdmod.dll, wmvdecod.dll
            decoders[1] = "{c9bfbccf-e60e-4588-a3df-5a03b1fd9585}"; // wvc1dmod.dll

            try
            {                
                InitializeComponent();

                this.TitleLabel.Text += System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();

                InitializeSettings();

                CheckCodecVersions(this.EncodersLabel, this.DecodersLabel);

                ReadEncoderSettings();

                RefreshUI(EncoderSettings);

                ReadDecoderSettings();

                RefreshUI(DecoderSettings);
            }
            catch (Exception ex)
            {
                MessageBox.Show("A fatal error occurred: " + ex.Message + "\n\nIn method: " + ex.TargetSite + "\n\nStack Trace:\n" + ex.StackTrace, "Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw ex;
            }
        }
       
        public void InitializeSettings()
        {   

            // Add registry settings in this way:
            // RegistryValueName, WMVSetting(CheckBoxObject, ValuesListObject, Enabled?, DefaultValue, ValueOffset, LowestProfileLevel)

            EncoderSettings.Add("Compression Optimization Type", new WMVSetting(CompOptCheck, CompOptList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("DenoiseOption", new WMVSetting(DenoiseCheck, DenoiseList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Dquant Option", new WMVSetting(DquantOptionCheck, DquantOptionList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Dquant Strength", new WMVSetting(DquantStrengthCheck, DquantStrengthList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Force B Frame Delta QP", new WMVSetting(BDeltaQPCheck, BDeltaQPList, false, Int32.MinValue, 0, vc1profile.MP));
            EncoderSettings.Add("Force Encoding Width", new WMVSetting(WidthCheck, WidthList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Force Encoding Height", new WMVSetting(HeightCheck, HeightList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Force LoopFilter", new WMVSetting(LoopFilterCheck, LoopFilterList, false, Int32.MinValue, 0, vc1profile.MP));
            EncoderSettings.Add("Force Median", new WMVSetting(MedianCheck, MedianList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Force NoiseEdgeRemoval", new WMVSetting(NoiseEdgeCheck, NoiseEdgeList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Force NumThreads", new WMVSetting(ThreadsCheck, ThreadsList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Force Overlap", new WMVSetting(OverlapCheck, OverlapList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Force Video Scaling", new WMVSetting(ScalingCheck, ScalingList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Lookahead", new WMVSetting(LookaheadCheck, LookaheadList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Macroblock Mode Cost Method", new WMVSetting(MBModeCostCheck, MBModeCostList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Motion Match Method", new WMVSetting(MMatchCheck, MMatchList, false, Int32.MinValue, 1, vc1profile.SP));
            EncoderSettings.Add("Motion Search Level", new WMVSetting(MSLevelCheck, MSLevelList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Motion Search Range", new WMVSetting(MSRangeCheck, MSRangeList, false, Int32.MinValue, 1, vc1profile.MP));
            EncoderSettings.Add("Motion Vector Coding Method", new WMVSetting(MVCodingCheck, MVCodingList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Motion Vector Cost Method", new WMVSetting(MVCostCheck, MVCostList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("NumBFrames", new WMVSetting(NumBFramesCheck, NumBFramesList, false, Int32.MinValue, 0, vc1profile.MP));
            EncoderSettings.Add("Perceptual Option", new WMVSetting(PercOptCheck, PercOptList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("VideoType", new WMVSetting(VideoTypeCheck, VideoTypeList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("Lookahead Ratecontrol", new WMVSetting(LRCCheck, LRCList, false, Int32.MinValue, 0, vc1profile.SP));
            EncoderSettings.Add("Encoder Thread Affinity", new WMVSetting(ThreadAffinityCheck, ThreadAffinityList, false, Int32.MinValue, 0, vc1profile.SP));

            EncoderSettings.Add("LumaRedux", new WMVSetting(RangeReduxCheck, LumaReduxList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("LumaExp", new WMVSetting(RangeReduxCheck, LumaExpansionList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("ChromaRedux", new WMVSetting(RangeReduxCheck, ChromaReduxList, false, Int32.MinValue, 0, vc1profile.AP));
            EncoderSettings.Add("ChromaExp", new WMVSetting(RangeReduxCheck, ChromaExpansionList, false, Int32.MinValue, 0, vc1profile.AP));

            DecoderSettings.Add("HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\WMVideo\\DXVA", new WMVSetting(DXVACheck, DXVAList, false, Int32.MinValue, 0, vc1profile.SP));
            DecoderSettings.Add("HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\WMVideo\\DXVATRACE", new WMVSetting(DXVATraceCheck, DXVATraceList, false, Int32.MinValue, 0, vc1profile.SP));
            DecoderSettings.Add("HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\Deinterlace", new WMVSetting(DeinterlacerCheck, DeinterlacerList, false, Int32.MinValue, 0, vc1profile.SP));
            DecoderSettings.Add("HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\Post Process Mode", new WMVSetting(PostProcCheck, PostProcList, false, Int32.MinValue, 1, vc1profile.SP));
            DecoderSettings.Add("HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\Force Post Process Mode", new WMVSetting(PostProcCheck, PostProcList, false, Int32.MinValue, 1, vc1profile.SP));

        }

        public string GetCOMNameAndVersion(string GUID)
        {
            RegistryKey CLSIDkey, WMSDKMeritKey;
            string DLLname, WMSDKMerit;
            System.Diagnostics.FileVersionInfo DLLversion;
            string COMNameAndVersion = "";
            Boolean x86OS;
            

            try
            {
                x86OS = System.Environment.GetEnvironmentVariable("PROCESSOR_ARCHITECTURE").ToLower() == "x86";

                if (x86OS)
                {
                    CLSIDkey = Registry.ClassesRoot.OpenSubKey("CLSID\\" + GUID + "\\InprocServer32");
                    WMSDKMeritKey = Registry.ClassesRoot.OpenSubKey("CLSID\\" + GUID);
                }
                else
                {
                    CLSIDkey = Registry.ClassesRoot.OpenSubKey("Wow6432Node\\CLSID\\" + GUID + "\\InprocServer32");
                    WMSDKMeritKey = Registry.ClassesRoot.OpenSubKey("Wow6432Node\\CLSID\\" + GUID);
                }
                
            }
            catch
            {
                CLSIDkey = null;
                WMSDKMeritKey = null;
            }        
            
            if (CLSIDkey != null && WMSDKMeritKey != null)
            {
                if (CLSIDkey.GetValue(null) != null)
                    DLLname = CLSIDkey.GetValue(null).ToString();
                else DLLname = "";

                if (WMSDKMeritKey.GetValue("WMSDKMerit") != null)
                    WMSDKMerit = WMSDKMeritKey.GetValue("WMSDKMerit").ToString();
                else WMSDKMerit = "";

                if (DLLname != "")
                {
                    COMNameAndVersion = DLLname.Substring(DLLname.LastIndexOf('\\') + 1);

                    try
                    {
                        DLLversion = System.Diagnostics.FileVersionInfo.GetVersionInfo(DLLname);
                        COMNameAndVersion += ",  " + DLLversion.FileMajorPart + "." + DLLversion.FileMinorPart + "." + DLLversion.FileBuildPart + "." + DLLversion.FilePrivatePart;
                    }
                    catch { }

                    if (WMSDKMerit != "")
                        COMNameAndVersion += ",  Priority: " + WMSDKMerit;
                }
            }

            return COMNameAndVersion;
        }

        public void CheckCodecVersions(Label enclabel, Label declabel)
        {
            string nameversion = "";
            Int32 encodermask = 0;
            
            enclabel.Text = "";
            declabel.Text = "";

            // list all registered encoders
            for (Int32 i = 0; i < encoders.Length; i++ )
            {
                nameversion = GetCOMNameAndVersion(encoders[i]);

                if (nameversion != "")
                {
                    enclabel.Text = enclabel.Text + nameversion + "\n";

                    // only count wmv3 and wvc1 encoders
                    if (nameversion.ToLower().IndexOf("wmvadve.dll") < 0)
                        encodermask += (Int32)Math.Pow(2,i);
                }
            }

            // list all registered decoders
            for (Int32 j = 0; j < decoders.Length; j++)
            {
                nameversion = GetCOMNameAndVersion(decoders[j]);

                if (nameversion != "")
                    declabel.Text = declabel.Text + nameversion + "\n";
            }

            switch (encodermask)
            {
                case 0:
                case 1:
                    MessageBox.Show("No configurable WMV9 encoders were detected on your system.\nConfiguring encoder settings will have no effect on WMV9 encoding.", "Warning:", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case 2:
                case 3:
                    MessageBox.Show("Only a configurable WMV9 Advanced Profile encoder was detected on your system.\nConfiguring encoder settings will have no effect on WMV9 Simple and Main Profile\nencoding.", "Warning:", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }

            if (encodermask >= 8)
            {
                PercOptList.Items.AddRange(new object[] {
                    "Aggressive ADZ 1",
                    "Aggressive ADZ 2",
                    "Aggressive ADZ 3",
                    "Aggressive ADZ 4",
                    "Aggressive ADZ 5"});

                MSLevelList.Items.AddRange(new object[] {
                    "Adaptive Integer Chroma",
                    "Adaptive True Chroma"});
            }
            if (encodermask >= 16)
                MinimumSize = new Size(940, 570);
            else
            {
                EnhancementsGroup.Hide();
                MaximumSize = new Size(940, 520);
            }

        }

        public void ReadEncoderSettings()
        {

            string scrunchkey = "Software\\Microsoft\\Scrunch\\WMVideo\\";
            Int32 regvalue = Int32.MinValue;
            Int32 rangeredux = Int32.MinValue;

            RegistryKey scrunch = Registry.CurrentUser.OpenSubKey(scrunchkey);

            if (scrunch != null)
            {
                rangeredux = (Int32)scrunch.GetValue("Force Range Reduction", Int32.MinValue);

                foreach (KeyValuePair<string, WMVSetting> UI in EncoderSettings)
                {
                    regvalue = (Int32)scrunch.GetValue(UI.Key.ToString(), Int32.MinValue);

                    if (regvalue != Int32.MinValue)
                    {
                        switch(UI.Key.ToString())
                        {
                            case "Force NumThreads":
                                UI.Value.value = regvalue / 2;
                                UI.Value.enabled = true;
                                break;

                            case "Perceptual Option":
                                try // ensure values out of range don't access out of index range
                                {
                                    regvalue = MapPercOptToADZ[regvalue];
                                }
                                catch
                                {
                                    regvalue = 0;
                                }
                                goto default;

                            default:
                                UI.Value.value = regvalue + UI.Value.offset;
                                UI.Value.enabled = true;
                                break;
                        }
                    }
                    else UI.Value.enabled = false;

                    if (rangeredux != Int32.MinValue)
                    {
                        switch (UI.Key.ToString())
                        {
                            case "LumaRedux":
                                UI.Value.value = rangeredux / 16777216;
                                UI.Value.enabled = true;
                                break;
                            case "LumaExp":
                                UI.Value.value = (rangeredux / 65536) % 8;
                                UI.Value.enabled = true;
                                break;
                            case "ChromaRedux":
                                UI.Value.value = (rangeredux / 256) % 8;
                                UI.Value.enabled = true;
                                break;
                            case "ChromaExp":
                                UI.Value.value = rangeredux % 8;
                                UI.Value.enabled = true;
                                break;
                        }
                    }

                    // Set presets

                    if (UI.Key.ToString() == "Compression Optimization Type")
                    {
                        switch (UI.Value.value)
                        {
                            case 1:
                                EncoderSettings["Motion Search Level"].value = 1;
                                EncoderSettings["Motion Match Method"].value = 0;
                                EncoderSettings["Dquant Option"].value = 2;
                                EncoderSettings["Motion Search Range"].value = 0;
                                EncoderSettings["NumBFrames"].value = 1;
                                EncoderSettings["Motion Vector Cost Method"].value = 1;
                                EncoderSettings["Force LoopFilter"].value = 1;
                                break;
                        }
                    }

                }
            }

        }

        public void ReadDecoderSettings()
        {
                        
            Int32 regvalue = Int32.MinValue;

            string regkey = "";
            string regvaluename = "";

            foreach (KeyValuePair<string, WMVSetting> UI in DecoderSettings)
            {
                regkey = UI.Key.Substring(0, UI.Key.LastIndexOf('\\'));
                regvaluename = UI.Key.Substring(UI.Key.LastIndexOf('\\') + 1);
                try
                {
                    regvalue = (Int32)Registry.GetValue(regkey, regvaluename, Int32.MinValue);
                }
                catch
                {
                    regvalue = Int32.MinValue;
                }


                if (regvalue != Int32.MinValue)
                {
                    UI.Value.value = regvalue + UI.Value.offset;
                    UI.Value.enabled = true;
                }
                else UI.Value.enabled = false;
            }

        }

        public void SaveEncoderSettings()
        {
            string scrunchkey = "Software\\Microsoft\\Scrunch\\WMVideo\\";
            RegistryKey scrunch;
            Int32 regvalue = 0;
            Int32 rangeredux = 0;
            ComboBox list;
            NumericUpDown updown;

            try
            {
                scrunch = Registry.CurrentUser.CreateSubKey(scrunchkey);
            }
            catch
            {
                scrunch = null;
            }
                

            if (scrunch != null)
            {
                foreach (KeyValuePair<string, WMVSetting> UI in EncoderSettings)
                {
                    if (UI.Value.checkname.Checked)
                    {
                        regvalue = 0;

                        switch (UI.Value.listname.GetType().Name)
                        {
                            case "ComboBox":
                                list = (ComboBox)UI.Value.listname;
                                if (list.SelectedIndex >= 0)
                                    regvalue = list.SelectedIndex - UI.Value.offset;
                                else
                                    regvalue = 0;
                                break;
                            case "NumericUpDown":
                                updown = (NumericUpDown)UI.Value.listname;
                                regvalue = (Int32)updown.Value - UI.Value.offset;
                                break;
                        }

                        switch (UI.Key.ToString())
                        {
                            case "LumaRedux":
                                rangeredux = rangeredux + regvalue * 16777216;
                                break;
                            case "LumaExp":
                                rangeredux = rangeredux + regvalue * 65536;
                                break;
                            case "ChromaRedux":
                                rangeredux = rangeredux + regvalue * 256;
                                break;
                            case "ChromaExp":
                                rangeredux = rangeredux + regvalue;
                                break;
                            case "Perceptual Option":
                                regvalue = MapADZToPercOpt[regvalue];
                                goto default;
                            case "Force NumThreads":
                                regvalue = (Int32)Math.Pow(2, regvalue);
                                goto default;
                            default:
                                try
                                {
                                    scrunch.SetValue(UI.Key.ToString(), regvalue, RegistryValueKind.DWord);
                                }
                                catch { }
                                break;
                        }
                    }
                    else
                    {
                        scrunch.DeleteValue(UI.Key.ToString(),false);
                    }
                }

                if (rangeredux > 0)
                {
                    try
                    {
                        scrunch.SetValue("Force Range Reduction", rangeredux, RegistryValueKind.DWord);
                    }
                    catch { }
                }
                else
                    scrunch.DeleteValue("Force Range Reduction", false);

            }

        }


        public void SaveDecoderSettings()
        {
            string regkey = "";
            string regvaluename = "";
            Int32 regvalue = 0;
            ComboBox list;
            NumericUpDown updown;
            RegistryKey key;

            foreach (KeyValuePair<string, WMVSetting> UI in DecoderSettings)
            {

                regkey = UI.Key.Substring(0, UI.Key.LastIndexOf('\\'));
                regkey = regkey.Substring(regkey.IndexOf('\\') + 1);
                regvaluename = UI.Key.Substring(UI.Key.LastIndexOf('\\') + 1);

                try
                {
                    key = Registry.CurrentUser.CreateSubKey(regkey);
                }
                catch
                {
                    key = null;
                }

                if (key != null)
                {
                    if (UI.Value.checkname.Checked)
                    {
                        regvalue = 0;

                        switch (UI.Value.listname.GetType().Name)
                        {
                            case "ComboBox":
                                list = (ComboBox)UI.Value.listname;
                                if (list.SelectedIndex >= 0)
                                    regvalue = list.SelectedIndex - UI.Value.offset;
                                else
                                    regvalue = 0;
                                break;
                            case "NumericUpDown":
                                updown = (NumericUpDown)UI.Value.listname;
                                regvalue = (Int32)updown.Value - UI.Value.offset;
                                break;
                        }

                        try
                        {
                            key.SetValue(regvaluename, regvalue, RegistryValueKind.DWord);
                        }
                        catch { }
                    }
                    else
                    {
                        key.DeleteValue(regvaluename, false);
                    }
                }
            }

           
        }


        public void ExportEncoderSettings()
        {

            Int32 regvalue = 0;
            Int32 rangeredux = 0;
            ComboBox list;
            NumericUpDown updown;
            SaveFileDialog exportfiledlg = new SaveFileDialog();
            TextWriter exportfile;

            exportfiledlg.DefaultExt = "reg";
            exportfiledlg.AddExtension = true;
            exportfiledlg.Filter = "Registration files (*.reg)|*.reg|All files (*.*)|*.*";
            exportfiledlg.OverwritePrompt = true;
            
            if (exportfiledlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    exportfile = new StreamWriter(exportfiledlg.FileName);
                }
                catch
                {
                    MessageBox.Show("Could not create file " + exportfiledlg.FileName, "Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    exportfile = null;
                }

                if (exportfile != null)
                {
                    exportfile.WriteLine("Windows Registry Editor Version 5.00");
                    exportfile.WriteLine();
                    exportfile.WriteLine("[HKEY_CURRENT_USER\\Software\\Microsoft\\Scrunch\\WMVideo]");

                    foreach (KeyValuePair<string, WMVSetting> UI in EncoderSettings)
                    {

                        if (UI.Value.checkname.Checked)
                        {
                            regvalue = 0;

                            switch (UI.Value.listname.GetType().Name)
                            {
                                case "ComboBox":
                                    list = (ComboBox)UI.Value.listname;
                                    if (list.SelectedIndex >= 0)
                                        regvalue = list.SelectedIndex - UI.Value.offset;
                                    else
                                        regvalue = 0;
                                    break;
                                case "NumericUpDown":
                                    updown = (NumericUpDown)UI.Value.listname;
                                    regvalue = (Int32)updown.Value - UI.Value.offset;
                                    break;
                            }

                            switch (UI.Key.ToString())
                            {
                                case "LumaRedux":
                                    rangeredux = rangeredux + regvalue * 16777216;
                                    break;
                                case "LumaExp":
                                    rangeredux = rangeredux + regvalue * 65536;
                                    break;
                                case "ChromaRedux":
                                    rangeredux = rangeredux + regvalue * 256;
                                    break;
                                case "ChromaExp":
                                    rangeredux = rangeredux + regvalue;
                                    break;
                                case "Perceptual Option":
                                    regvalue = MapADZToPercOpt[regvalue];
                                    goto default;
                                case "Force NumThreads":
                                    regvalue = (Int32)Math.Pow(2, regvalue);
                                    goto default;
                                default:
                                    exportfile.WriteLine("\"" + UI.Key.ToString() + "\"=dword:" + String.Format("{0:x8}", regvalue));
                                    break;
                            }
                        }
                        else
                            switch (UI.Key.ToString())
                            {
                                case "LumaRedux": break;
                                case "LumaExp": break;
                                case "ChromaRedux": break;
                                case "ChromaExp": break;
                                default:
                                    exportfile.WriteLine("\"" + UI.Key.ToString() + "\"=-");
                                    break;
                            }
                    }

                    if (rangeredux > 0)
                        exportfile.WriteLine("\"Force Range Reduction\"=dword:" + String.Format("{0:x8}", rangeredux));
                    else
                        exportfile.WriteLine("\"Force Range Reduction\"=-");

                    exportfile.Close();
                }

            }


        }

        public void ImportEncoderSettings()
        {
            OpenFileDialog importfiledlg;
            System.Diagnostics.Process regimport;

            importfiledlg = new OpenFileDialog();

            importfiledlg.DefaultExt = "reg";
            importfiledlg.Filter = "Registration files (*.reg)|*.reg|All files (*.*)|*.*";
            importfiledlg.AddExtension = false;
            importfiledlg.CheckFileExists = true;
            importfiledlg.CheckPathExists = true;

            if (importfiledlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    regimport = System.Diagnostics.Process.Start("regedit", "/s \"" + importfiledlg.FileName + "\"");

                    while (!regimport.HasExited) ;
                }
                catch
                {
                    MessageBox.Show("Error importing registration file " + importfiledlg.FileName, "Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                ReadEncoderSettings();

                RefreshUI(EncoderSettings);
            }
            
        }

        public void RefreshUI(Dictionary<string, WMVSetting> settings)
        {
            Type controltype;
            ComboBox list;
            NumericUpDown updown;

            ProfileList.SelectedIndex = -1;

            foreach (KeyValuePair<string, WMVSetting> UI in settings)
            {
                UI.Value.checkname.Checked = UI.Value.listname.Enabled = UI.Value.enabled;

                UI.Value.checkname.Enabled = true;

                controltype = UI.Value.listname.GetType();

                if (UI.Value.enabled | UI.Value.value > Int32.MinValue)
                {
                    switch (controltype.Name)
                    {
                        case "ComboBox":
                            list = (ComboBox)UI.Value.listname;
                            if (UI.Value.value < 0)
                                list.SelectedIndex = 0;
                            else if (UI.Value.value >= list.Items.Count)
                                list.SelectedIndex = list.Items.Count - 1;
                            else
                                list.SelectedIndex = UI.Value.value;
                            break;
                        case "NumericUpDown":
                            updown = (NumericUpDown)UI.Value.listname;
                            if (UI.Value.value < updown.Minimum)
                                updown.Value = updown.Minimum;
                            else if (UI.Value.value > updown.Maximum)
                                updown.Value = updown.Maximum;
                            else
                                updown.Value = UI.Value.value;
                            break;
                    }

                }
                else
                {
                    switch (controltype.Name)
                    {
                        case "ComboBox":
                            list = (ComboBox)UI.Value.listname;
                            list.SelectedIndex = -1;
                            break;
                        case "NumericUpDown":
                            updown = (NumericUpDown)UI.Value.listname;
                            updown.Value = updown.Minimum;
                            break;
                    }
                }
            }
        }
              

        private void DenoiseCheck_CheckedChanged(object sender, EventArgs e)
        {
            DenoiseList.Enabled = DenoiseCheck.Checked;
            if (DenoiseCheck.Checked & DenoiseList.SelectedIndex == -1)
                DenoiseList.SelectedIndex = 0;
        }

        private void LoopFilterCheck_CheckedChanged(object sender, EventArgs e)
        {
            LoopFilterList.Enabled = LoopFilterCheck.Checked;
            if (LoopFilterCheck.Checked & LoopFilterList.SelectedIndex == -1)
                LoopFilterList.SelectedIndex = 0;
        }

        private void MedianCheck_CheckedChanged(object sender, EventArgs e)
        {
            MedianList.Enabled = MedianCheck.Checked;
            if (MedianCheck.Checked & MedianList.SelectedIndex == -1)
                MedianList.SelectedIndex = 0;
        }

        private void CompOptCheck_CheckedChanged(object sender, EventArgs e)
        {
            CompOptList.Enabled = CompOptCheck.Checked;
            if (CompOptCheck.Checked & CompOptList.SelectedIndex == -1)
                CompOptList.SelectedIndex = 0;
        }

        private void DquantOptionCheck_CheckedChanged(object sender, EventArgs e)
        {
            DquantOptionList.Enabled = DquantOptionCheck.Checked;
            if (DquantOptionCheck.Checked & DquantOptionList.SelectedIndex == -1)
                DquantOptionList.SelectedIndex = 0;
        }

        private void DquantStrengthCheck_CheckedChanged(object sender, EventArgs e)
        {
            DquantStrengthList.Enabled = DquantStrengthCheck.Checked;
            if (DquantStrengthCheck.Checked & DquantStrengthList.SelectedIndex == -1)
                DquantStrengthList.SelectedIndex = 0;
        }

        private void ScalingCheck_CheckedChanged(object sender, EventArgs e)
        {
            ScalingList.Enabled = ScalingCheck.Checked;
            if (ScalingCheck.Checked & ScalingList.SelectedIndex == -1)
                ScalingList.SelectedIndex = 0;
        }

        private void PercOptCheck_CheckedChanged(object sender, EventArgs e)
        {
            PercOptList.Enabled = PercOptCheck.Checked;
            if (PercOptCheck.Checked & PercOptList.SelectedIndex == -1)
                PercOptList.SelectedIndex = 0;
        }

        private void OverlapCheck_CheckedChanged(object sender, EventArgs e)
        {
            OverlapList.Enabled = OverlapCheck.Checked;
            if (OverlapCheck.Checked & OverlapList.SelectedIndex == -1)
                OverlapList.SelectedIndex = 0;
        }

        private void NoiseEdgeCheck_CheckedChanged(object sender, EventArgs e)
        {
            NoiseEdgeList.Enabled = NoiseEdgeCheck.Checked;
            if (NoiseEdgeCheck.Checked & NoiseEdgeList.SelectedIndex == -1)
                NoiseEdgeList.SelectedIndex = 0;
        }

        private void MSLevelCheck_CheckedChanged(object sender, EventArgs e)
        {
            MSLevelList.Enabled = MSLevelCheck.Checked;
            if (MSLevelCheck.Checked & MSLevelList.SelectedIndex == -1)
                MSLevelList.SelectedIndex = 0;
        }

        private void MSRangeCheck_CheckedChanged(object sender, EventArgs e)
        {
            MSRangeList.Enabled = MSRangeCheck.Checked;
            if (MSRangeCheck.Checked & MSRangeList.SelectedIndex == -1)
                MSRangeList.SelectedIndex = 1;
        }

        private void MMatchCheck_CheckedChanged(object sender, EventArgs e)
        {
            MMatchList.Enabled = MMatchCheck.Checked;
            if (MMatchCheck.Checked & MMatchList.SelectedIndex == -1)
                MMatchList.SelectedIndex = 1;
        }

        private void MBModeCostCheck_CheckedChanged(object sender, EventArgs e)
        {
            MBModeCostList.Enabled = MBModeCostCheck.Checked;
            if (MBModeCostCheck.Checked & MBModeCostList.SelectedIndex == -1)
                MBModeCostList.SelectedIndex = 1;
        }

        private void MVCodingCheck_CheckedChanged(object sender, EventArgs e)
        {
            MVCodingList.Enabled = MVCodingCheck.Checked;
            if (MVCodingCheck.Checked & MVCodingList.SelectedIndex == -1)
                MVCodingList.SelectedIndex = 0;
        }

        private void MVCostCheck_CheckedChanged(object sender, EventArgs e)
        {
            MVCostList.Enabled = MVCostCheck.Checked;
            if (MVCostCheck.Checked & MVCostList.SelectedIndex == -1)
                MVCostList.SelectedIndex = 1;
        }

        private void NumBFramesCheck_CheckedChanged(object sender, EventArgs e)
        {
            NumBFramesList.Enabled = NumBFramesCheck.Checked;
        }

        private void BDeltaQPCheck_CheckedChanged(object sender, EventArgs e)
        {
            BDeltaQPList.Enabled = BDeltaQPCheck.Checked;
        }

        private void LookaheadCheck_CheckedChanged(object sender, EventArgs e)
        {
            LookaheadList.Enabled = LookaheadCheck.Checked;
        }

        private void WidthCheck_CheckedChanged(object sender, EventArgs e)
        {
            WidthList.Enabled = WidthCheck.Checked;
        }

        private void HeightCheck_CheckedChanged(object sender, EventArgs e)
        {
            HeightList.Enabled = HeightCheck.Checked;
        }

        private void ThreadsCheck_CheckedChanged(object sender, EventArgs e)
        {
            ThreadsList.Enabled = ThreadsCheck.Checked;
            if (ThreadsCheck.Checked & ThreadsList.SelectedIndex == -1)
                ThreadsList.SelectedIndex = 0;
        }

        private void VideoTypeCheck_CheckedChanged(object sender, EventArgs e)
        {
            VideoTypeList.Enabled = VideoTypeCheck.Checked;
            if (VideoTypeCheck.Checked & VideoTypeList.SelectedIndex == -1)
                VideoTypeList.SelectedIndex = 0;
        }

        private void RangeReduxCheck_CheckedChanged(object sender, EventArgs e)
        {
            ChromaExpansionList.Enabled = RangeReduxCheck.Checked;
            ChromaReduxList.Enabled = RangeReduxCheck.Checked;
            LumaExpansionList.Enabled = RangeReduxCheck.Checked;
            LumaReduxList.Enabled = RangeReduxCheck.Checked;
        }

        private void CompOptList_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (CompOptList.SelectedIndex)
            {
                case 1:
                    MSLevelList.SelectedIndex = 1;
                    MMatchList.SelectedIndex = 0; // -1
                    DquantOptionList.SelectedIndex = 2;
                    MSRangeList.SelectedIndex = 0; // -1
                    NumBFramesList.Value = 1;
                    MVCostList.SelectedIndex = 1;
                    LoopFilterList.SelectedIndex = 1;

                    MSLevelList.Enabled = MMatchList.Enabled = DquantOptionList.Enabled = MSRangeList.Enabled = NumBFramesList.Enabled = MVCostList.Enabled = LoopFilterList.Enabled = false;
                    MSLevelCheck.Checked = MMatchCheck.Checked = DquantOptionCheck.Checked = MSRangeCheck.Checked = NumBFramesCheck.Checked = MVCostCheck.Checked = LoopFilterCheck.Checked = false;

                    break;
            }
        }

        private void DXVACheck_CheckedChanged(object sender, EventArgs e)
        {
            DXVAList.Enabled = DXVACheck.Checked;
            if (DXVACheck.Checked & DXVAList.SelectedIndex == -1)
                DXVAList.SelectedIndex = 0;
        }

        private void DXVATraceCheck_CheckedChanged(object sender, EventArgs e)
        {
            DXVATraceList.Enabled = DXVATraceCheck.Checked;
            if (DXVATraceCheck.Checked & DXVATraceList.SelectedIndex == -1)
                DXVATraceList.SelectedIndex = 0;
        }

        private void DeinterlacerCheck_CheckedChanged(object sender, EventArgs e)
        {
            DeinterlacerList.Enabled = DeinterlacerCheck.Checked;
            if (DeinterlacerCheck.Checked & DeinterlacerList.SelectedIndex == -1)
                DeinterlacerList.SelectedIndex = 0;
        }

        private void PostProcCheck_CheckedChanged(object sender, EventArgs e)
        {
            PostProcList.Enabled = PostProcCheck.Checked;
            if (PostProcCheck.Checked & PostProcList.SelectedIndex == -1)
                PostProcList.SelectedIndex = 0;
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            ReadEncoderSettings();

            RefreshUI(EncoderSettings);
        }

        private void QuitButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ApplyButton_Click(object sender, EventArgs e)
        {
            SaveEncoderSettings();
        }

        private void ExportButton_Click(object sender, EventArgs e)
        {
            ExportEncoderSettings();
        }

        private void ImportButton_Click(object sender, EventArgs e)
        {
            ImportEncoderSettings();
        }

        private void ClearButton_Click(object sender, EventArgs e)
        {
            foreach (KeyValuePair<string, WMVSetting> UI in EncoderSettings)
            {
                UI.Value.checkname.Checked = UI.Value.listname.Enabled = false;
            }
        }

        private void QuitDButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ClearDButton_Click(object sender, EventArgs e)
        {
            foreach (KeyValuePair<string, WMVSetting> UI in DecoderSettings)
            {
                UI.Value.checkname.Checked = UI.Value.listname.Enabled = false;
            }
        }

        private void RefreshDButton_Click(object sender, EventArgs e)
        {
            ReadDecoderSettings();

            RefreshUI(DecoderSettings);
        }

        private void ApplyDButton_Click(object sender, EventArgs e)
        {
            SaveDecoderSettings();
        }

        private void ProfileButton_Click(object sender, EventArgs e)
        {
            if (ProfileList.SelectedIndex >= 0)
            {
                foreach (KeyValuePair<string, WMVSetting> UI in EncoderSettings)
                {
                    if (ProfileList.SelectedIndex > (Int32)UI.Value.profile)
                        UI.Value.checkname.Enabled = UI.Value.checkname.Checked = false;
                    else
                        UI.Value.checkname.Enabled = true;
                }

            }
        }

        private void WebsiteLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(WebsiteLabel.Text);
            } catch {
                MessageBox.Show("Could not open web link!", "Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ContactLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start("mailto:" + ContactLabel.Text);
            }
            catch
            {
                MessageBox.Show("Could not open e-mail link!", "Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void LicenseLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            LicenseBox license = new LicenseBox();

            license.ShowDialog();

            license.Dispose();

            license.Close();
        }

        private void LRCCheck_CheckedChanged(object sender, EventArgs e)
        {
            LRCList.Enabled = LRCCheck.Checked;
            if (LRCCheck.Checked) LookaheadCheck.Checked = false;
        }

        private void ThreadAffinityCheck_CheckedChanged(object sender, EventArgs e)
        {
            ThreadAffinityList.Enabled = ThreadAffinityCheck.Checked;
            if (ThreadAffinityCheck.Checked & ThreadAffinityList.SelectedIndex == -1)
                ThreadAffinityList.SelectedIndex = 1;
        }

                
    }

    public class WMVSetting
    {
        
        public CheckBox checkname;
        public Control listname;
        public Boolean enabled;
        public Int32 value;
        public Int32 offset;
        public vc1profile profile;
        
        public WMVSetting(CheckBox cbCheckName, Control cListName, Boolean bEnabled, Int32 iValue, Int32 iOffset, vc1profile vProfile)
        {
            checkname = cbCheckName;
            listname = cListName;
            enabled = bEnabled;
            value = iValue;
            offset = iOffset;
            profile = vProfile;
        }
    }
}